# print_tools version 1.1motdb
# examples:
# from tools.print_tools import *
# from tools.print_tools import pplog
from pprint import PrettyPrinter

# ansicolors and some symbols
###################################################################################################

# Reset
RST="\033[0m"           # Text Reset

# Regular Colors
Black="\033[0;30m"      # Black
Red="\033[0;31m"        # Red
Green="\033[0;32m"      # Green
Yellow="\033[0;33m"     # Yellow
Blue="\033[0;34m"       # Blue
Purple="\033[0;35m"     # Purple
Cyan="\033[0;36m"       # Cyan
White="\033[0;37m"      # White

# Bold
BBlack="\033[1;30m"     # Black
BRed="\033[1;31m"       # Red
BGreen="\033[1;32m"     # Green
BYellow="\033[1;33m"    # Yellow
BBlue="\033[1;34m"      # Blue
BPurple="\033[1;35m"    # Purple
BCyan="\033[1;36m"      # Cyan
BWhite="\033[1;37m"     # White

# Background
BGBlack="\033[40m"      # Black
BGRed="\033[41m"        # Red
BGGreen="\033[42m"      # Green
BGYellow="\033[43m"     # Yellow
BGBlue="\033[44m"       # Blue
BGPurple="\033[45m"     # Purple
BGCyan="\033[46m"       # Cyan
BGWhite="\033[47m"      # White

# Underline
ULBlack="\033[4;30m"      # Black
ULRed="\033[4;31m"        # Red
ULGreen="\033[4;32m"      # Green
ULYellow="\033[4;33m"     # Yellow
ULBlue="\033[4;34m"       # Blue
ULPurple="\033[4;35m"     # Purple
ULCyan="\033[4;36m"       # Cyan
ULWhite="\033[4;37m"      # White

# Symbols
checkmark='\u2713'
crossmark='\u2717'

# cursor
mvup="\033[A"
mvdown="\033[B"
clearln="\33[2K\r"



# Functions
###################################################################################################
width, indent, margin = 50, 4, 0
from pprint import PrettyPrinter
pr = print

def pp(msg='', w=0, i=0):
    w = w or width
    i = i or indent
    _pr = PrettyPrinter( indent=i, width=w ).pprint
    _pr( msg )

def pf(msg='', w=0, i=0):
    w = w or width
    i = i or indent
    _pf = PrettyPrinter( indent=i, width=w ).pformat
    return _pf( msg )

def ppd(o=None, w=0, i=0):
    if isinstance(o,dict):
        _o = [ k for k,v in o.items() if not k.startswith('_') ]
    elif isinstance(o,(list,tuple,set)):
        _o = [ i for i in list(o) if not i.startswith('_') ]
    elif o:
        _o = [ i for i in dir(o) if not i.startswith('_') ]
    pp( _o, w, i )

def newsprint(mls='', w=40, m=0, i=0, return_txt=False):
    w = w or width
    i = i or indent
    m = m or margin
    """ Prints Multi-line-strings (mls) as newsprint
        - inserts newlines accoring to width (w) and spaces according to margin (m).
        - steps
            1. Drop in mls/multi-line-string = ''' blah blah blah '''
            2. run newsprint(mls) or newsprint(mls,80) etc
        - example:
            mls = '''
            - steps
                1. Drop in mls/multi-line-string = ``` blah blah blah ```
                2. run newsprint(mls) or newsprint(mls,30,2) etc
            '''
            newsprint(mls,30,2) =>
              - steps
              1. Drop in
              mls/multi-line-string =
              ``` blah blah blah ```
              2. run newsprint(mls)
              or newsprint(mls,30,2)
              etc
    """
    newspf = PrettyPrinter( indent=i, width=w ).pformat
    strcols = newspf( mls )
    strcols = strcols.strip()
    if strcols[-1] == ')':
        strcols = strcols[:-1]
    if strcols[0] == '(':
        strcols = strcols[1:]
    if not return_txt:
        for line in strcols.splitlines():
            newsline = line.replace("'","").replace('\\n','').rstrip()
            newsline = " "*m + newsline
            print(newsline)
    else:
        news_list = []
        for line in strcols.splitlines():
            newsline = line.replace("'","").replace('\\n','').rstrip()
            newsline = " "*m + newsline
            news_list.append(newsline)
        return '\n'.join(news_list)

def pplog(msg='', log_lvl=0, title='', w=88, i=4, logger=None):
    debug = 0 if not 'debug' in locals() else debug
    if log_lvl > debug:
        return
    log = logger or print
    w = w or width
    i = i or indent
    if title:
        log("\n"+"*"*w)
        leftspcs = int(0.5*(w - len(title)))
        log(f"{' '*leftspcs}{title}")
        log("*"*w)
    if not msg:
        # only log title
        return
    _pf = PrettyPrinter( indent=i, width=w ).pformat
    #logmsg = _pf( msg )
    logmsg = msg if not logger else _pf( msg )
    #logmsg = logmsg.strip().strip('(').strip(')')
    logmsg = logmsg.strip('(').strip(')')
    for line in logmsg.splitlines():
        #line = re.sub( re.escape('\n'), '', line )
        print(f'{line[:w]}\n')
        remains = line[w:]
        while remains:
            dbl = i*2
            log(f'{" "*dbl}{remains[:w-dbl]}')
            remains = remains[w-dbl:]

import re
def strip_ansi(strs):
    re_ansi = re.compile(r'\x1b[^m]*m')
    if isinstance(strs, str):
        cleaned = re_ansi.sub('', strs)
    elif isinstance(strs, (list,tuple)):
        cleaned = [ strip_ansi(s) for s in strs ]
        if isinstance(strs,tuple):
            cleaned = tuple(cleaned)
    elif isinstance(strs, set):
        cleaned = { strip_ansi(s) for s in strs }
    elif isinstance(strs, dict):
        cleaned = { strip_ansi(k):strip_ansi(v) for k,v in strs }
    else:
        cleaned = strs
    return cleaned


exec_mainpr = '''
def mainpr(self, msg):
    """ if __name__ == '__main__': print(msg) """
    if __name__ == '__main__':
        print(msg)
'''

exec_mainpp = '''
def mainpp(self, msg):
    """ if __name__ == '__main__': pp(msg) """
    if __name__ == '__main__':
        pp(msg)
'''

import re
def left_shift_ss(ss=''):
    """ Counts spaces at beginning of each line
        so SQL Statement (ss) can be printed
        left shifted to margin
    """
    sslns = [ ln for ln in ss.splitlines() if ln.strip() ]
    ct = min( re.match(' *',ln).span()[-1] for ln in sslns )
    ss = '\n'.join( ln[ct:] if ln[0:ct] == ' '*ct else ln for ln in sslns )
    print(f'\n{ss}\n')
    return ss


# batteries included imports
from collections import namedtuple
from pathlib import Path as P
from types import SimpleNamespace

# pypi imports

# custom imports
from .tools.cli_tools import Cli, is_venv, get_venv_path
from .tools.path_tools import resolve, resolve_dir, parent
from .tools.load_tools import load_toml, load_cfg
        

class DB:
  """ Base Database Class """
  def __init__(self, **kw):
    cfg = load_cfg( kw.get('cfg') )
    self.type = kw.get('type') or cfg.get('default', {}).get('type')

  def test(self):
    print(f"I'm {self.type}")


class DB_sqlite3(DB):
  def __init__(self, **kw):
    super().__init__(**kw)
    self.type = 'sqlite3'


class DB_mysql(DB):
  def __init__(self, **kw):
    super().__init__(**kw)
    self.type = 'mysql'


class DB_factory:
  def __init__(self, db_type='sqlite3', **kw):
    
    match db_type:
      case 'sqlite3':
        _db = DB_sqlite3(**kw)
      case 'mysql':
        _db = DB_mysql(**kw)
      case _:
        _db = DB_sqlite3(**kw)
    
    # inherit not private method overrides from specific db instances
    self.__dict__.update( { k:v for k,v in _db.__dict__.items() if not k.startswith('_') } )

    ''' Sample dbx.toml
    [default]
    type = 'sqlite3'
    mysql_port = 3306
    mssql_port = 1433
    user = 'commonly_defined_username'

    [sqlite3]
    host1.user = 'some_username_in_db@host1' # accessed as cfg.get('sqlite3.host')
    '''

  def get_db(self, db_type=None):
    db_type = db_type or self.type
    self.__init__(db_type)
    return self


class Dbx(DB):
  def __init__(self, **kw):
    super().__init__(**kw)
    # set cfg/params
    cfg = globals().get('cfg') or {}
    self.cfg = kw['cfg'] = load_cfg( kw.get('cfg') or cfg ) # cfg=dict or cfg=path-to-.toml
    db_type = kw.get('type') or self.cfg.get('type')
    db_type = db_type or self.cfg.get('default',{}).get('type') or 'sqlite3'
     
    # inherit non-private method overrides from specific db instances
    _DB = DB_factory( db_type, **kw )
    self.update_dbx( _DB )

    default = self.cfg.get('default') or {}
    if self.cfg.get(self.type) and isinstance( self.cfg.get(self.type), dict ):
      print('here')
      self.update_dbx( self.cfg.get(self.type) )
    elif isinstance( default, dict ):
      self.update_dbx( default )


  def update_dbx(self, dict_prms):
    # set non-private method overrides
    _dict_prms = dict_prms if not hasattr(dict_prms,'__dict__') else dict_prms.__dict__
    self.__dict__.update( { k:v for k,v in _dict_prms.items() if not k.startswith('_') } )
    


import sys
sys.path.insert(0,'/Users/mfsteele/Git/motdb/src')
#import motdb
from motdb.tools.print_tools import pp, ppd
from motdb import Dbx

db = Dbx()
db1 = Dbx(cfg='dbx.toml')
db2 = Dbx(cfg='s.toml')


pp(f'{db.type=}')
pp(f'{db1.type=}')
pp(f'{db2.type=}')



# load_tools version 0.1.1motdb
# batteries included imports
import sys
from pathlib import Path as P
from types import SimpleNamespace
from typing import Any

# pypi imports
import tomli

# custom imports
from .cli_tools import is_venv, get_venv_path
from .path_tools import parent


# load toml
def load_toml(f=''):
  dict_cfg = {}
  try:

    path_toml = P(f)
    if not path_toml.is_file():

      if is_venv():
        venv = get_venv_path()
        venv_parent = parent(venv)
        path_parent = P(venv_parent)
        path_cfgs = [ i.name for i in path_parent.iterdir() if i.name == 'dbx.toml' ]
        if path_cfgs:
          with open(path_cfgs[0], 'rb') as f:
            dict_cfg = tomli.load(f)
      else:
        curr_path = P(sys.path[0])
        path_cfgs = [ i.name for i in curr_path.iterdir() if i.name == 'dbx.toml' ]
        if path_cfgs:
          with open(path_cfgs[0], 'rb') as f:
            dict_cfg = tomli.load(f)
    else:
      with open(f, 'rb') as f:
        dict_cfg = tomli.load(f)
    
  except Exception as e:
      print(str(e))

  finally:
    return dict_cfg
  

def load_cfg(cfg=None):
  dict_cfg = {}
  try:
    if not cfg:
      dict_cfg = load_toml()
    elif isinstance(cfg,str):
      dict_cfg = load_toml(cfg)
    elif isinstance(cfg, dict):
      dict_cfg = cfg
    else:
      dict_cfg = {}
  except Exception as e:
    print(str(e))
  finally:
    return dict_cfg
  



